==> sssd_2.10.1.orig.tar.gz: sssd-2.10.1/src/tests/tests/system/tests/test_infopipe.py <==
"""
Infopipe tests.

:requirement: Service/IFP
"""

from __future__ import annotations

import pytest
from sssd_test_framework.roles.client import Client
from sssd_test_framework.roles.generic import GenericProvider
from sssd_test_framework.topology import KnownTopology, KnownTopologyGroup


@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__get_user_properties(client: Client, provider: GenericProvider):
    """
    :title: Get a user's property
    :setup:
        1. Add 'user-1' to the provider
        2. Launch SSSD
    :steps:
        1. Call FindByName() to get the user's object path
        2. Create a new object associated to that user using its object path
        3. Get the UID property using Get()
        4. Get the GID property with direct access
        5. Get all the properties
    :expectedresults:
        1. User is found and object path retrieved
        2. The object is created
        3. Property is returned and has the right value
        4. Property is returned and has the right value
        5. Some sample properties are present and have the right value
    :customerscenario: False
    """
    provider.user("user-1").add(uid=10001, gid=20001)

    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")
    user_path = users.FindByName("user-1")
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10001" == user_path
    ), "FindByName('user-1') returned an unexpected user path"

    user = client.ifp.getObject(user_path)
    uid = user.Get("org.freedesktop.sssd.infopipe.Users.User", "uidNumber")
    assert uid == 10001, "user.Get(user_path, 'uidNumber') returned an unexpected UID"

    gid = user.gidNumber
    assert gid == 20001, "user.gidNumber returned an unexpected GID"

    props = user.GetAll("org.freedesktop.sssd.infopipe.Users.User")
    assert props["uidNumber"] == 10001, "GetAll(Users.User) returned an unexpected uidNumber"
    assert props["gidNumber"] == 20001, "GetAll(Users.User) returned an unexpected gidNumber"
    assert props["name"] == "user-1", "GetAll(Users.User) returned an unexpected name"
    assert props["homeDirectory"] == "/home/user-1", "GetAll(Users.User) returned an unexpected homeDirectory"


@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__get_domain_properties(client: Client):
    """
    :title: Access a domain's information through InfoPipe
    :setup:
        1. Launch SSSD
    :steps:
        1. Call IsOnline()
        2. Call GetAll() with "org.freedesktop.sssd.infopipe.Domains"
        3. Read the property enumerable
    :expectedresults:
        1. Domain is online
        2. All data is returned
        3. Is enumerable
    :customerscenario: False
    """
    client.sssd.start()

    domain = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Domains/test")

    res = domain.IsOnline()
    assert isinstance(res, bool), "IsOnline() didn't return a boolean"
    assert res, "IsOnline() returned false"

    res = domain.GetAll("org.freedesktop.sssd.infopipe.Domains")
    assert isinstance(res, dict), "GetAll(infopipe.Domains) didn't return a dict"
    assert res["name"] == "test", "GetAll(infopipe.Domains) returned an unexpected name"
    assert res["provider"] == "ldap", "GetAll(infopipe.Domains) returned an unexpected provider"
    assert (
        res["login_format"] == r"^((?P<name>.+)@(?P<domain>[^@]+)|(?P<name>[^@]+))$"
    ), "GetAll(infopipe.Domains) returned an unexpected login_format"
    assert isinstance(
        res["primary_servers"], list
    ), "GetAll(infopipe.Domains) returned a primary_servers which is not a list"
    assert (
        len(res["primary_servers"]) == 1
    ), "GetAll(infopipe.Domains) returned a wrong number of elements in primary_servers"
    assert (
        res["primary_servers"][0] == "ldap://master.ldap.test"
    ), "GetAll(infopipe.Domains) returned a wrong URI in primary_servers"

    res = domain.enumerable
    assert isinstance(res, bool), "Property enumerable didn't return a boolean"
    assert not res, "Property enumerable returned the wrong value"


@pytest.mark.ticket(gh=6020, bz=2128840, jira="SSSD-5054")
@pytest.mark.topology(KnownTopology.IPA)
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__list_by_attr(client: Client, provider: GenericProvider):
    """
    :title: Test InfoPipe.Users.ListByAttr()
    :setup:
        1. Add 'user-1' to the provider
        2. Add 'user-2' to the provider
        3. Add 'user-3' to the provider
        4. Add "user_attributes = +extraName" to the [ifp] configuration section
        5. Add "ldap_user_extra_attrs = extraName:uid" to the 'test' domain configuration
        6. Create the 'app' application domain inheriting from 'test'
        7. Launch SSSD
    :steps:
        1. Call ListByAttr() with "extraName=user-2" and no limit
        2. Call ListByAttr() with "extraName=user-*" and no limit
        3. Call ListByAttr() with "extraName=user-*" and a limit of 4
        4. Call ListByAttr() with "extraName=nouser-*" and no limit
        5. Call ListByAttr() with "noName=*" and no limit
    :expectedresults:
        1. User is found exactly once for 'test' and once for 'app'
        2. All three users are found on each domain
        3. Four of the six user entries are returned
        4. No user is returned
        5. No user is returned
    :customerscenario: False
    """
    provider.user("user-1").add(uid=10001)
    provider.user("user-2").add(uid=10002)
    provider.user("user-3").add(uid=10003)

    cfg = client.sssd.config
    cfg.set("sssd", "domains", cfg.get("sssd", "domains") + ",app")
    cfg.add_section("ifp")
    cfg.set("ifp", "user_attributes", "+extraName")
    cfg.set("domain/test", "ldap_user_extra_attrs", "extraName:uid")
    cfg.add_section("application/app")
    cfg.set("application/app", "inherit_from", "test")

    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")

    list = users.ListByAttr("extraName", "user-2", 0)
    assert len(list) == 2, "ListByAttr('extraName', 'user-2', 0) returned a wrong number of elements"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in list
    ), "ListByAttr('extraName', 'user-2', 0) is missing element test/user-2"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d2_40app" in list
    ), "ListByAttr('extraName', 'user-2', 0) is missing element app/user-2"

    list = users.ListByAttr("extraName", "user-*", 0)
    assert len(list) == 6, "Wrong number of elements in list"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10001" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element test/10001"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element test/10002"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10003" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element test/10003"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d1_40app" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element app/user-1"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d2_40app" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element app/user-2"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d3_40app" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element app/user-3"

    model = list  # We'll use this list as model for the next check
    list = users.ListByAttr("extraName", "user-*", 4)
    assert len(list) == 4, "ListByAttr('extraName', 'user-*', 4) returned a wrong number of elements"
    # We don't know which two will be returned
    for user in list:
        assert user in model, "ListByAttr('extraName', 'user-*', 4) returned an unexpected element"

    list = users.ListByAttr("extraName", "nouser-*", 0)
    assert len(list) == 0, "ListByAttr('extraName', 'nouser-*', 0) returned unexpected elements"

    list = users.ListByAttr("noattr", "*", 0)
    assert len(list) == 0, "ListByAttr('noattr', '*', 0) returned unexpected elements"


@pytest.mark.ticket(gh=[6360, 6361], jira="SSSD-5054")
@pytest.mark.topology(KnownTopology.IPA)
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__list_by_name(client: Client, provider: GenericProvider):
    """
    :title: Test InfoPipe.Users.ListByName()
    :setup:
        1. Add 'user-1' to the provider
        2. Add 'user-2' to the provider
        3. Add 'user-3' to the provider
        4. Launch SSSD
    :steps:
        1. Call ListByName() with "user-2" and no limit
        2. Call ListByName() with "user-*" and no limit
        3. Call ListByName() with "user-*" and a limit of 2
        4. Call ListByName() with "nouser-*" and no limit
    :expectedresults:
        1. User is found and none else
        2. All three users are found
        3. Two of the three users are returned
        4. No user is returned
    :customerscenario: False
    """
    provider.user("user-1").add(uid=10001)
    provider.user("user-2").add(uid=10002)
    provider.user("user-3").add(uid=10003)
    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")

    result = users.ListByName("user-2", 0)
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in result
    ), "ListByName('user-2', 0) returned unexpected element"
    assert len(result) == 1, "ListByName('user-2', 0) returned a wrong number of elements"

    result = users.ListByName("user-*", 0)
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10001" in result
    ), "ListByName('user-*', 0) is missing element 10001"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in result
    ), "ListByName('user-*', 0) is missing element 10002"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10003" in result
    ), "ListByName('user-*', 0) is missing element 10003"
    assert len(result) == 3, "ListByName('user-*', 0) returned a wrong number of elements"

    model = result  # We'll use this list as model for the next check
    result = users.ListByName("user-*", 2)
    assert len(result) == 2, "ListByName('user-*', 2) returned a wrong number of elements"
    # We don't know which two will be returned
    for user in result:
        assert user in model, "ListByName('user-*', 2) returned an unexpected element"

    result = users.ListByName("nouser*", 0)
    assert len(result) == 0, "ListByName('nouser*', 0) returned unexpected elements"


@pytest.mark.importance("medium")
@pytest.mark.ticket(bz=1667252)
@pytest.mark.topology(KnownTopologyGroup.AnyProvider)
def test_infopipe__lookup_user_with_extra_attributes(client: Client, provider: GenericProvider):
    """
    :title: Infopipe does not crash looking up extra attribute
    :setup:
        1. Create user "user1"
        2. Enable infopipe, add a test attribute and start SSSD
    :steps:
        1. Lookup user using sssctl
        2. Check SSSD service
    :expectedresults:
        1. User found
        2. Service is running
    :customerscenario: True
    """
    provider.user("user1").add()
    client.sssd.sssd["services"] = "nss, pam, ifp"
    client.sssd.domain["ldap_user_extra_attrs"] = "test:homeDirectory"
    client.sssd.ifp["user_attributes"] = "+test"
    client.sssd.start()

    result = client.sssctl.user_checks("user1")
    assert result.rc == 0, "User not found!"

    result = client.sssd.svc.status("sssd")
    assert result.rc == 0, "Service is not running!"

==> sssd_2.11.1.orig.tar.gz: sssd-2.11.1/src/tests/tests/system/tests/test_infopipe.py <==
"""
Infopipe tests.

:requirement: Service/IFP
"""

from __future__ import annotations

import pytest
from sssd_test_framework.roles.client import Client
from sssd_test_framework.roles.generic import GenericProvider
from sssd_test_framework.roles.ldap import LDAP
from sssd_test_framework.topology import KnownTopology, KnownTopologyGroup


@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__get_user_properties(client: Client, provider: GenericProvider):
    """
    :title: Get a user's property
    :setup:
        1. Add 'user-1' to the provider
        2. Launch SSSD
    :steps:
        1. Call FindByName() to get the user's object path
        2. Create a new object associated to that user using its object path
        3. Get the UID property using Get()
        4. Get the GID property with direct access
        5. Get all the properties
    :expectedresults:
        1. User is found and object path retrieved
        2. The object is created
        3. Property is returned and has the right value
        4. Property is returned and has the right value
        5. Some sample properties are present and have the right value
    :customerscenario: False
    """
    provider.user("user-1").add(uid=10001, gid=20001)

    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")
    user_path = users.FindByName("user-1")
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10001" == user_path
    ), "FindByName('user-1') returned an unexpected user path"

    user = client.ifp.getObject(user_path)
    uid = user.Get("org.freedesktop.sssd.infopipe.Users.User", "uidNumber")
    assert uid == 10001, "user.Get(user_path, 'uidNumber') returned an unexpected UID"

    gid = user.gidNumber
    assert gid == 20001, "user.gidNumber returned an unexpected GID"

    props = user.GetAll("org.freedesktop.sssd.infopipe.Users.User")
    assert props["uidNumber"] == 10001, "GetAll(Users.User) returned an unexpected uidNumber"
    assert props["gidNumber"] == 20001, "GetAll(Users.User) returned an unexpected gidNumber"
    assert props["name"] == "user-1", "GetAll(Users.User) returned an unexpected name"
    assert props["homeDirectory"] == "/home/user-1", "GetAll(Users.User) returned an unexpected homeDirectory"


@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__get_domain_properties(client: Client):
    """
    :title: Access a domain's information through InfoPipe
    :setup:
        1. Launch SSSD
    :steps:
        1. Call IsOnline()
        2. Call GetAll() with "org.freedesktop.sssd.infopipe.Domains"
        3. Read the property enumerable
    :expectedresults:
        1. Domain is online
        2. All data is returned
        3. Is enumerable
    :customerscenario: False
    """
    client.sssd.start()

    domain = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Domains/test")

    res = domain.IsOnline()
    assert isinstance(res, bool), "IsOnline() didn't return a boolean"
    assert res, "IsOnline() returned false"

    res = domain.GetAll("org.freedesktop.sssd.infopipe.Domains")
    assert isinstance(res, dict), "GetAll(infopipe.Domains) didn't return a dict"
    assert res["name"] == "test", "GetAll(infopipe.Domains) returned an unexpected name"
    assert res["provider"] == "ldap", "GetAll(infopipe.Domains) returned an unexpected provider"
    assert (
        res["login_format"] == r"^((?P<name>.+)@(?P<domain>[^@]+)|(?P<name>[^@]+))$"
    ), "GetAll(infopipe.Domains) returned an unexpected login_format"
    assert isinstance(
        res["primary_servers"], list
    ), "GetAll(infopipe.Domains) returned a primary_servers which is not a list"
    assert (
        len(res["primary_servers"]) == 1
    ), "GetAll(infopipe.Domains) returned a wrong number of elements in primary_servers"
    assert (
        res["primary_servers"][0] == "ldap://master.ldap.test"
    ), "GetAll(infopipe.Domains) returned a wrong URI in primary_servers"

    res = domain.enumerable
    assert isinstance(res, bool), "Property enumerable didn't return a boolean"
    assert not res, "Property enumerable returned the wrong value"


@pytest.mark.ticket(gh=6020, bz=2128840, jira="SSSD-5054")
@pytest.mark.topology(KnownTopology.IPA)
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__list_by_attr(client: Client, provider: GenericProvider):
    """
    :title: Test InfoPipe.Users.ListByAttr()
    :setup:
        1. Add 'user-1' to the provider
        2. Add 'user-2' to the provider
        3. Add 'user-3' to the provider
        4. Add "user_attributes = +extraName" to the [ifp] configuration section
        5. Add "ldap_user_extra_attrs = extraName:uid" to the 'test' domain configuration
        6. Create the 'app' application domain inheriting from 'test'
        7. Launch SSSD
    :steps:
        1. Call ListByAttr() with "extraName=user-2" and no limit
        2. Call ListByAttr() with "extraName=user-*" and no limit
        3. Call ListByAttr() with "extraName=user-*" and a limit of 4
        4. Call ListByAttr() with "extraName=nouser-*" and no limit
        5. Call ListByAttr() with "noName=*" and no limit
    :expectedresults:
        1. User is found exactly once for 'test' and once for 'app'
        2. All three users are found on each domain
        3. Four of the six user entries are returned
        4. No user is returned
        5. No user is returned
    :customerscenario: False
    """
    provider.user("user-1").add(uid=10001)
    provider.user("user-2").add(uid=10002)
    provider.user("user-3").add(uid=10003)

    cfg = client.sssd.config
    cfg.set("sssd", "domains", cfg.get("sssd", "domains") + ",app")
    cfg.add_section("ifp")
    cfg.set("ifp", "user_attributes", "+extraName")
    cfg.set("domain/test", "ldap_user_extra_attrs", "extraName:uid")
    cfg.add_section("application/app")
    cfg.set("application/app", "inherit_from", "test")

    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")

    list = users.ListByAttr("extraName", "user-2", 0)
    assert len(list) == 2, "ListByAttr('extraName', 'user-2', 0) returned a wrong number of elements"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in list
    ), "ListByAttr('extraName', 'user-2', 0) is missing element test/user-2"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d2_40app" in list
    ), "ListByAttr('extraName', 'user-2', 0) is missing element app/user-2"

    list = users.ListByAttr("extraName", "user-*", 0)
    assert len(list) == 6, "Wrong number of elements in list"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10001" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element test/10001"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element test/10002"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10003" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element test/10003"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d1_40app" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element app/user-1"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d2_40app" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element app/user-2"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/app/user_2d3_40app" in list
    ), "ListByAttr('extraName', 'user-*', 0) is missing element app/user-3"

    model = list  # We'll use this list as model for the next check
    list = users.ListByAttr("extraName", "user-*", 4)
    assert len(list) == 4, "ListByAttr('extraName', 'user-*', 4) returned a wrong number of elements"
    # We don't know which two will be returned
    for user in list:
        assert user in model, "ListByAttr('extraName', 'user-*', 4) returned an unexpected element"

    list = users.ListByAttr("extraName", "nouser-*", 0)
    assert len(list) == 0, "ListByAttr('extraName', 'nouser-*', 0) returned unexpected elements"

    list = users.ListByAttr("noattr", "*", 0)
    assert len(list) == 0, "ListByAttr('noattr', '*', 0) returned unexpected elements"


@pytest.mark.ticket(gh=[6360, 6361], jira="SSSD-5054")
@pytest.mark.topology(KnownTopology.IPA)
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__list_by_name(client: Client, provider: GenericProvider):
    """
    :title: Test InfoPipe.Users.ListByName()
    :setup:
        1. Add 'user-1' to the provider
        2. Add 'user-2' to the provider
        3. Add 'user-3' to the provider
        4. Launch SSSD
    :steps:
        1. Call ListByName() with "user-2" and no limit
        2. Call ListByName() with "user-*" and no limit
        3. Call ListByName() with "user-*" and a limit of 2
        4. Call ListByName() with "nouser-*" and no limit
    :expectedresults:
        1. User is found and none else
        2. All three users are found
        3. Two of the three users are returned
        4. No user is returned
    :customerscenario: False
    """
    provider.user("user-1").add(uid=10001)
    provider.user("user-2").add(uid=10002)
    provider.user("user-3").add(uid=10003)
    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")

    result = users.ListByName("user-2", 0)
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in result
    ), "ListByName('user-2', 0) returned unexpected element"
    assert len(result) == 1, "ListByName('user-2', 0) returned a wrong number of elements"

    result = users.ListByName("user-*", 0)
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10001" in result
    ), "ListByName('user-*', 0) is missing element 10001"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10002" in result
    ), "ListByName('user-*', 0) is missing element 10002"
    assert (
        "/org/freedesktop/sssd/infopipe/Users/test/10003" in result
    ), "ListByName('user-*', 0) is missing element 10003"
    assert len(result) == 3, "ListByName('user-*', 0) returned a wrong number of elements"

    model = result  # We'll use this list as model for the next check
    result = users.ListByName("user-*", 2)
    assert len(result) == 2, "ListByName('user-*', 2) returned a wrong number of elements"
    # We don't know which two will be returned
    for user in result:
        assert user in model, "ListByName('user-*', 2) returned an unexpected element"

    result = users.ListByName("nouser*", 0)
    assert len(result) == 0, "ListByName('nouser*', 0) returned unexpected elements"


@pytest.mark.importance("medium")
@pytest.mark.ticket(bz=1667252)
@pytest.mark.topology(KnownTopologyGroup.AnyProvider)
def test_infopipe__lookup_user_with_extra_attributes(client: Client, provider: GenericProvider):
    """
    :title: Infopipe does not crash looking up extra attribute
    :setup:
        1. Create user "user1"
        2. Enable infopipe, add a test attribute and start SSSD
    :steps:
        1. Lookup user using sssctl
        2. Check SSSD service
    :expectedresults:
        1. User found
        2. Service is running
    :customerscenario: True
    """
    provider.user("user1").add()
    client.sssd.sssd["services"] = "nss, pam, ifp"
    client.sssd.domain["ldap_user_extra_attrs"] = "test:homeDirectory"
    client.sssd.ifp["user_attributes"] = "+test"
    client.sssd.start()

    result = client.sssctl.user_checks("user1")
    assert result.rc == 0, "User not found!"

    result = client.sssd.svc.status("sssd")
    assert result.rc == 0, "Service is not running!"


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__lookup_group_and_properties(client: Client, provider: GenericProvider):
    """
    :title: Infopipe lookup group, members and properties
    :setup:
        1. Create users and group
        2. Add users to groups
        3. Start SSSD
    :steps:
        1. Find group object
        2. Get GID from group object
        3. Get all properties from object
    :expectedresults:
        1. Group object is found
        2. GID found and matches expected value
        3. Properties are returned with correct members and properties
    :customerscenario: False
    """
    u1 = provider.user("user1").add(uid=10001)
    u2 = provider.user("user2").add(uid=10002)
    provider.group("group-1").add(gid=30001).add_members([u1, u2])
    client.sssd.start()

    groups = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Groups")
    group_path = groups.FindByName("group-1")
    group = client.ifp.getObject(group_path)

    gid = group.Get("org.freedesktop.sssd.infopipe.Groups.Group", "gidNumber")
    assert gid == 30001, f"Expected GID 30001, got {gid}"

    # Initially the list is empty and to update the groups UpdateMemberList needs to be issued
    group.UpdateMemberList()

    props = group.GetAll("org.freedesktop.sssd.infopipe.Groups.Group")

    assert props["name"] == "group-1", f"Unexpected name: {props['name']}"
    assert props["gidNumber"] == 30001, f"Unexpected gidNumber: {props['gidNumber']}"

    assert len(props["users"]) == 2, f"Expected 2 members, got {len(props['users'])}"
    assert any("10001" in str(u) for u in props["users"]), "user1 (uid=10001) not found"
    assert any("10002" in str(u) for u in props["users"]), "user2 (uid=10002) not found"


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__lookup_returns_latest_group_membership(client: Client, provider: GenericProvider):
    """
    :title: Infopipe returns latest group changes
    :setup:
       1. Create user and two groups
       2. Add user to first group
       3. Start SSSD
    :steps:
       1. Lookup user’s group
       2. Move user to the other group
       3. Lookup user’s group
    :expectedresults:
       1. User is the first group
       2. User is moved to second group
       3. User is the second group
    :customerscenario: False
    """
    user = provider.user("user1").add(uid=10001)
    group1 = provider.group("group1").add(gid=20001)
    group2 = provider.group("group2").add(gid=20002)
    group1.add_member(user)

    client.sssd.start()

    users_iface = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")
    groups_iface = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Groups")

    user_path = users_iface.FindByName("user1")
    user_obj = client.ifp.getObject(user_path)

    group1_path = groups_iface.FindByName("group1")
    initial_groups = user_obj.groups

    assert len(initial_groups) == 2, f"Expected 2 groups initially (primary + group1), got {len(initial_groups)}"
    assert group1_path in initial_groups, "user1 not in group1 initially"

    group1.remove_member(user)
    group2.add_member(user)

    client.sssd.restart(clean=True)

    user_path = users_iface.FindByName("user1")
    user_obj = client.ifp.getObject(user_path)

    group2_path = groups_iface.FindByName("group2")
    updated_groups = user_obj.groups

    assert len(updated_groups) == 2, f"Expected 2 groups after update (primary + group2), got {len(updated_groups)}"
    assert group2_path in updated_groups, "user1 not in group2 after update"
    assert group1_path not in updated_groups, "user1 still in group1 after removal"


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
def test_infopipe__lookup_user_attributes(client: Client, provider: LDAP):
    """
    :title: Infopipe looks up user attributes
    :setup:
       1. Create user with ‘sn’, ‘givenName” and ‘mail’ attributes set
       2. Configure sssd with ‘parameter to fetch custom attributes = sn, givenName, mail’
       3. Start SSSD
    :steps:
       1. Look up user attributes
    :expectedresults:
       1. All requested attributes are returned with expected values
    :customerscenario: False
    """
    provider.user("user1").add(uid=10001, gid=10001, sn="Test", givenName="User", mail="user1@example.com")

    client.sssd.domain["ldap_user_extra_attrs"] = "sn:sn,givenName:givenName,mail:mail"
    client.sssd.ifp["user_attributes"] = "+sn,+givenName,+mail"
    client.sssd.start()

    users = client.ifp.getObject("/org/freedesktop/sssd/infopipe/Users")
    user_path = users.FindByName("user1")
    user = client.ifp.getObject(user_path)
    props = user.GetAll("org.freedesktop.sssd.infopipe.Users.User")
    user_attrs = props["extraAttributes"]
    assert "sn" in user_attrs and user_attrs["sn"] == ["Test"], "Expected sn to be ['Test']"
    assert "givenName" in user_attrs and user_attrs["givenName"] == ["User"], "Expected givenName to be ['User']"
    assert "mail" in user_attrs and user_attrs["mail"] == [
        "user1@example.com"
    ], "Expected mail to be ['user1@example.com']"
